--------------------------------------------------------------------------------
--  
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

entity Sistema_Base is
	port (
		clk 	: in std_logic;
		rst 	: in std_logic;
		sw		: in std_logic_vector(3 downto 0);
		leds	: out std_logic_vector(7 downto 0)
	);
end Sistema_Base;

architecture Behavioral of Sistema_Base is

	-- Memoria de programa
	component program
		generic(
			C_FAMILY : string := "S6"; 
			C_RAM_SIZE_KWORDS : integer := 1;
			C_JTAG_LOADER_ENABLE : integer := 0);
		Port (
			address : in std_logic_vector(11 downto 0);
			instruction : out std_logic_vector(17 downto 0);
			enable : in std_logic;
			rdl : out std_logic;                    
			clk : in std_logic
		);
	end component;

	-- Componente PicoBlaze
	component kcpsm6 
		generic(
			hwbuild : std_logic_vector(7 downto 0) := X"00";
			interrupt_vector : std_logic_vector(11 downto 0) := X"3FF";
			scratch_pad_memory_size : integer := 64
		);
		port (
			address : out std_logic_vector(11 downto 0);
			instruction : in std_logic_vector(17 downto 0);
			bram_enable : out std_logic;
			in_port : in std_logic_vector(7 downto 0);
			out_port : out std_logic_vector(7 downto 0);
			port_id : out std_logic_vector(7 downto 0);
			write_strobe : out std_logic;
			k_write_strobe : out std_logic;
			read_strobe : out std_logic;
			interrupt : in std_logic;
			interrupt_ack : out std_logic;
			sleep : in std_logic;
			reset : in std_logic;
			clk : in std_logic
		);
	end component;


	-- Senales usadas para conectar el KCPSM6

	signal              address : std_logic_vector(11 downto 0);
	signal          instruction : std_logic_vector(17 downto 0);
	signal          bram_enable : std_logic;
	signal              in_port : std_logic_vector(7 downto 0);
	signal             out_port : std_logic_vector(7 downto 0);
	signal              port_id : std_logic_vector(7 downto 0);
	signal         write_strobe : std_logic;
	signal       k_write_strobe : std_logic;
	signal          read_strobe : std_logic;
	signal            interrupt : std_logic := '0';
	signal        interrupt_ack : std_logic;
	signal         kcpsm6_sleep : std_logic;
	signal         kcpsm6_reset : std_logic;
	signal                  rdl : std_logic;
	
	signal write_to_leds   	: std_logic;
--	signal in_port_dummy	: std_logic_vector(7 downto 0) := (others => '0');

	signal rst_p : std_logic;

begin

	rst_p <= rst;

	-- Instanciacion del PicoBlaze y la memoria de programa
	-- No se utilizan interrupciones	
	processor: kcpsm6
		generic map (
			hwbuild => X"41",    -- 41 hex is ASCII character "A"
			interrupt_vector => X"3C0",   
			scratch_pad_memory_size => 64)
		port map(
			address => address,
			instruction => instruction,
			bram_enable => bram_enable,
			port_id => port_id,
			write_strobe => write_strobe,
			k_write_strobe => k_write_strobe,
			out_port => out_port,
			read_strobe => read_strobe,
			in_port => in_port,
			interrupt => interrupt,
			interrupt_ack => interrupt_ack,
			sleep => kcpsm6_sleep,
			reset => rst_p,
			clk => clk
		);

	-- Instanciacion de la memoria de programa
	program_rom: program
		generic map(
			C_FAMILY => "7S", 
			C_RAM_SIZE_KWORDS => 1,
			C_JTAG_LOADER_ENABLE => 1
		)
		port map( 
			address => address,      
			instruction => instruction,
			enable => bram_enable,
			rdl => rdl,
			clk => clk
		);
					   
  -- Unused signals tied off until required.
  -- Tying to other signals used to minimise warning messages.
  --
	kcpsm6_sleep <= write_strobe and k_write_strobe;  -- Always '0'

	
	-- Implementacion de la logica del puerto de salida
    --        leds_out (port 01)
	write_to_leds <= write_strobe and port_id(0);

	process(clk)
	begin  
		if rising_edge(clk) then
			if rst_p = '1'	then
				leds <= "00000000";
			elsif(write_to_leds = '1') then
				leds <= out_port;
			end if;
		end if;
	end process;


	--  Implementaion de la logica del puerto de entrada
	--  	switch_in (port 00)
	process(clk)
	begin  
		if rising_edge(clk) then
			case (port_id(0)) is
				when '0' =>
					in_port <= "0000" & sw;
				when others =>
					in_port <= (others => '0');
			end case;
		end if;
	end process;

end Behavioral;
